@@ -32,12 +32,12 @@ get_vars_indl <- function(x, indl, return = "data")
   oldClass(x) <- NULL
   if(is.list(value)) {
     oldClass(value) <- NULL # fastest ?? if(is.object(value)) oldClass(value) <- NULL ??
-    if(length(value[[1L]]) != length(x[[1L]])) stop("NROW(value) must match nrow(x)")
+    if(.Call(C_fnrow, value) != .Call(C_fnrow, x)) stop("NROW(value) must match nrow(x)")
     if(length(value) != length(ind)) stop("NCOL(value) must match selected variables") # length(num_vars(x))
     x[ind] <- value
     if(length(nam <- names(value))) names(x)[ind] <- nam #  == length(ind)
   } else {
-    if(NROW(unclass(value)) != length(x[[1L]])) stop("NROW(value) must match nrow(x)")
+    if(NROW(unclass(value)) != .Call(C_fnrow, x)) stop("NROW(value) must match nrow(x)")
     if(length(ind) != 1L) stop("NCOL(value) must match selected variables") # length(num_vars(x))
     x[[ind]] <- value
   }
@@ -113,14 +113,14 @@ logi_vars <- function(x, return = "data") get_vars_ind(x, .Call(C_vtypes, x, 0L)
 
 date_vars <- function(x, return = "data") get_vars_indl(x, vapply(`attributes<-`(x, NULL), is_date, TRUE), return)
 "date_vars<-" <- function(x, value) `get_vars_ind<-`(x, vapply(`attributes<-`(x, NULL), is_date, TRUE), value)
-Date_vars <- function(x, return = "data") {
-  message("Note that 'Date_vars' was renamed to 'date_vars'. It will not be removed anytime soon, but please use updated function names in new code, see help('collapse-renamed')")
-  date_vars(x, return)
-}
-"Date_vars<-" <- function(x, value) {
-   message("Note that 'Date_vars' was renamed to 'date_vars'. It will not be removed anytime soon, but please use updated function names in new code, see help('collapse-renamed')")
-  `date_vars<-`(x, value)
-}
+# Date_vars <- function(x, return = "data") {
+#   .Deprecated(msg = "'Date_vars' was renamed to 'date_vars'. It will be removed end of 2023, see help('collapse-renamed').")
+#   date_vars(x, return)
+# }
+# "Date_vars<-" <- function(x, value) {
+#   .Deprecated(msg = "'Date_vars' was renamed to 'date_vars'. It will be removed end of 2023, see help('collapse-renamed').")
+#   `date_vars<-`(x, value)
+# }
 
 cat_vars <- function(x, return = "data") get_vars_ind(x, .Call(C_vtypes, x, 1L) %!=% TRUE, return)
 "cat_vars<-" <- function(x, value) `get_vars_ind<-`(x, .Call(C_vtypes, x, 1L) %!=% TRUE, value)
@@ -180,25 +180,28 @@ gvr <- function(x, vars, return = "data", ...) {
   `get_vars_ind<-`(x, ind, value)
 }
 
-# Make faster ?
 "add_vars<-" <- function(x, pos = "end", value) {
   ax <- attributes(x)
   attributes(x) <- NULL
   lx <- length(x)
   if(is.list(value)) {
     oldClass(value) <- NULL # fastest ?
-    if(length(value[[1L]]) != length(x[[1L]])) stop("NROW(value) must match nrow(x)")
+    if(.Call(C_fnrow, value) != .Call(C_fnrow, x)) stop("NROW(value) must match nrow(x)")
     # res <- c(x, value)  # FASTER than commented out below
-    if(is.character(pos)) {
-      if(pos == "end") {
-        ax[["names"]] <- if(length(nam <- names(value)))  c(ax[["names"]], nam) else
-          c(ax[["names"]], paste0("V", seq(lx+1L, lx+length(value))))
-        return(condalcSA(c(x, value), ax, any(ax[["class"]] == "data.table")))
-      } else if(pos != "front") stop("pos needs to be 'end', 'front' or a suitable numeric / integer vector of positions!")
-        ax[["names"]] <- if(length(nam <- names(value)))  c(nam, ax[["names"]]) else
-          c(paste0("V", seq_along(value)), ax[["names"]])
-        return(condalcSA(c(value, x), ax, any(ax[["class"]] == "data.table")))
-    }
+    if(is.character(pos)) switch(pos,
+          end = {
+            ax[["names"]] <- if(length(nam <- names(value)))  c(ax[["names"]], nam) else
+              c(ax[["names"]], paste0("V", seq(lx+1L, lx+length(value))))
+            return(condalcSA(c(x, value), ax, any(ax[["class"]] == "data.table")))
+          },
+          front = {
+            ax[["names"]] <- if(length(nam <- names(value)))  c(nam, ax[["names"]]) else
+              c(paste0("V", seq_along(value)), ax[["names"]])
+            return(condalcSA(c(value, x), ax, any(ax[["class"]] == "data.table")))
+          },
+          stop("pos needs to be 'end', 'front' or a suitable numeric / integer vector of positions!")
+      )
+
     lv <- length(value)
     tl <- lv+lx
     if(!is.numeric(pos) || length(pos) != lv || bmax(pos) > tl) stop("pos needs to be 'end', 'front' or a suitable numeric / integer vector of positions!")
@@ -211,21 +214,25 @@ gvr <- function(x, vars, return = "data", ...) {
     # ax[["names"]] <- if(length(nam <- names(value)))  c(ax[["names"]], nam) else
     #   c(ax[["names"]], paste0("V", ind))
   } else {
-    if(NROW(value) != length(x[[1L]])) stop("NROW(value) must match nrow(x)")
+    if(NROW(value) != .Call(C_fnrow, x)) stop("NROW(value) must match nrow(x)")
     # res <- c(x, list(value)) # FASTER than below ? -> Nope
     # ax[["names"]] <- c(ax[["names"]], paste0("V", lx+1L))
     nam <- l1orlst(as.character(substitute(value)))
-    if(is.character(pos)) {
-      if(pos == "end") {
-        x[[lx+1L]] <- value
-        ax[["names"]] <- c(ax[["names"]], nam) # paste0("V", lx+1L)
-        return(condalcSA(x, ax, any(ax[["class"]] == "data.table")))
-      } else if(pos != "front") stop("pos needs to be 'end', 'front' or a suitable numeric / integer vector of positions!")
-      ax[["names"]] <- c(nam, ax[["names"]])
-      return(condalcSA(c(list(value), x), ax, any(ax[["class"]] == "data.table")))
-    }
+    if(is.character(pos)) switch(pos,
+         end = {
+           x[[lx+1L]] <- value
+           ax[["names"]] <- c(ax[["names"]], nam) # paste0("V", lx+1L)
+           return(condalcSA(x, ax, any(ax[["class"]] == "data.table")))
+         },
+         front = {
+           ax[["names"]] <- c(nam, ax[["names"]])
+           return(condalcSA(c(list(value), x), ax, any(ax[["class"]] == "data.table")))
+         },
+         stop("pos needs to be 'end', 'front' or a suitable numeric / integer vector of positions!")
+      )
+
     if(!is.numeric(pos) || length(pos) > 1L || pos > lx+1L) stop("pos needs to be 'end', 'front' or a suitable numeric / integer vector of positions!")
-    o <- forder.int(c(1:lx, pos-1L))
+    o <- forder.int(c(seq_len(lx), pos-1L))
     ax[["names"]] <- c(ax[["names"]], nam)[o]
     return(condalcSA(c(x, list(value))[o], ax, any(ax[["class"]] == "data.table")))
   }
@@ -233,10 +240,15 @@ gvr <- function(x, vars, return = "data", ...) {
 "av<-" <- `add_vars<-`
 
 add_vars <- function(x, ..., pos = "end") {
-  if(...length() == 1L) return(`add_vars<-`(x, pos, ...))
-  l <- c(...)
-  if(!all(fnrow2(x) == vlengths(l, FALSE))) stop("if multiple arguments are passed to '...', each needs to be a data.frame/list with column-lengths matching nrow(x)")
-  return(`add_vars<-`(x, pos, l)) # very minimal ! Doesn't work for vectors etc !
+  if(...length() == 1L) {
+    if(is.list(..1) || is.null(names(l <- list(...)))) return(`add_vars<-`(x, pos, ...))
+    return(`add_vars<-`(x, pos, l))
+  }
+  l <- list(...) # Old: c(...), did not allow atomic inputs...
+  l <- if(all(.Call(C_vtypes, l, 3L))) c(...) else # Checks if all is list...
+    unlist(lapply(l, function(z) if(is.list(z)) z else list(z)), recursive = FALSE)
+  if(!allv(vlengths(l, FALSE), fnrow(x))) stop("if multiple arguments are passed to '...', for all arguments NROW(arg) must match nrow(x)")
+  return(`add_vars<-`(x, pos, l))
 }
 av <- add_vars
 
