@@ -668,7 +668,7 @@ SEXP fsumlC(SEXP x, SEXP Rng, SEXP g, SEXP w, SEXP Rnarm, SEXP fill, SEXP Rdrop,
         #pragma omp parallel for num_threads(nthreads)
         for(int j = 0; j < l; ++j) fsum_g_omp_impl(px[j], DPTR(pout[j]), ng, pg, narm);
       } else {
-        for(int j = 0; j != l; ++j) pout[j] = fsum_g_impl(px[j], ng, pg, narm);
+        for(int j = 0; j != l; ++j) SET_VECTOR_ELT(out, j, fsum_g_impl(px[j], ng, pg, narm));
       }
     } else {
       double *restrict pw = REAL(w);
