@@ -1,6 +1,6 @@
 context("fvar and fsd")
 
-if(!is.null(attributes(identical(FALSE, TRUE)))) stop("OECD label issue")
+
 
 bvar <- stats::var
 bsd <- stats::sd
@@ -44,14 +44,8 @@ wvar <- function(x, w, na.rm = FALSE) {
   bsum(w*(x-weighted.mean(x,w))^2)/(bsum(w)-1)
 }
 
-wBY <- function(x, f, FUN, w, ...) {
-  if(is.atomic(x) && !is.array(x)) return(mapply(FUN, split(x, f), split(w, f), ...))
-  wspl <- split(w, f)
-  if(is.atomic(x)) return(dapply(x, function(xi) mapply(FUN, split(xi, f), wspl, ...)))
-  qDF(dapply(x, function(xi) mapply(FUN, split(xi, f), wspl, ...), return = "matrix"))
-}
 
-# fvar using Welford's Algoritm (default)
+# fvar using Welford's Algorithm (default)
 
 test_that("fvar performs like base::var", {
   expect_equal(fvar(NA), bvar(NA))
@@ -181,18 +175,18 @@ test_that("fvar with weights performs like wvar (defined above)", {
   expect_equal(fvar(mtcars, w = wdat, na.rm = FALSE), dapply(mtcars, wvar, wdat))
   expect_equal(fvar(mtcNA, w = wdat, na.rm = FALSE), dapply(mtcNA, wvar, wdat))
   expect_equal(fvar(mtcNA, w = wdat), dapply(mtcNA, wvar, wdat, na.rm = TRUE))
-  expect_equal(fvar(x, f, w), wBY(x, f, wvar, w))
-  expect_equal(fvar(x, f, w, na.rm = FALSE), wBY(x, f, wvar, w))
-  expect_equal(fvar(xNA, f, w, na.rm = FALSE), wBY(xNA, f, wvar, w))
-  expect_equal(na20(fvar(xNA, f, w)), na20(wBY(xNA, f, wvar, w, na.rm = TRUE)))
-  expect_equal(fvar(m, g, wdat), wBY(m, gf, wvar, wdat))
-  expect_equal(fvar(m, g, wdat, na.rm = FALSE), wBY(m, gf, wvar, wdat))
-  expect_equal(fvar(mNA, g, wdat, na.rm = FALSE),  wBY(mNA, gf, wvar, wdat))
-  expect_equal(na20(fvar(mNA, g, wdat)), na20(wBY(mNA, gf, wvar, wdat, na.rm = TRUE)))
-  expect_equal(fvar(mtcars, g, wdat), wBY(mtcars, gf, wvar, wdat))
-  expect_equal(fvar(mtcars, g, wdat, na.rm = FALSE), wBY(mtcars, gf, wvar, wdat))
-  expect_equal(fvar(mtcNA, g, wdat, na.rm = FALSE), wBY(mtcNA, gf, wvar, wdat))
-  expect_equal(na20(fvar(mtcNA, g, wdat)), na20(wBY(mtcNA, gf, wvar, wdat, na.rm = TRUE)))
+  expect_equal(fvar(x, f, w), BY(x, f, wvar, w))
+  expect_equal(fvar(x, f, w, na.rm = FALSE), BY(x, f, wvar, w))
+  expect_equal(fvar(xNA, f, w, na.rm = FALSE), BY(xNA, f, wvar, w))
+  expect_equal(na20(fvar(xNA, f, w)), na20(BY(xNA, f, wvar, w, na.rm = TRUE)))
+  expect_equal(fvar(m, g, wdat), BY(m, gf, wvar, wdat))
+  expect_equal(fvar(m, g, wdat, na.rm = FALSE), BY(m, gf, wvar, wdat))
+  expect_equal(fvar(mNA, g, wdat, na.rm = FALSE),  BY(mNA, gf, wvar, wdat))
+  expect_equal(na20(fvar(mNA, g, wdat)), na20(BY(mNA, gf, wvar, wdat, na.rm = TRUE)))
+  expect_equal(fvar(mtcars, g, wdat), BY(mtcars, gf, wvar, wdat))
+  expect_equal(fvar(mtcars, g, wdat, na.rm = FALSE), BY(mtcars, gf, wvar, wdat))
+  expect_equal(fvar(mtcNA, g, wdat, na.rm = FALSE), BY(mtcNA, gf, wvar, wdat))
+  expect_equal(na20(fvar(mtcNA, g, wdat)), na20(BY(mtcNA, gf, wvar, wdat, na.rm = TRUE)))
   # missing weights
   expect_equal(fvar(NA, w = NA), wvar(NA, NA))
   expect_equal(fvar(NA, w = NA, na.rm = FALSE), wvar(NA, NA))
@@ -215,18 +209,18 @@ test_that("fvar with weights performs like wvar (defined above)", {
   expect_equal(fvar(mtcars, w = wdatNA, na.rm = FALSE), dapply(mtcars, wvar, wdatNA))
   expect_equal(fvar(mtcNA, w = wdatNA, na.rm = FALSE), dapply(mtcNA, wvar, wdatNA))
   expect_equal(fvar(mtcNA, w = wdatNA), dapply(mtcNA, wvar, wdatNA, na.rm = TRUE))
-  expect_equal(na20(fvar(x, f, wNA)), na20(wBY(x, f, wvar, wNA, na.rm = TRUE)))
-  expect_equal(fvar(x, f, wNA, na.rm = FALSE), wBY(x, f, wvar, wNA))
-  expect_equal(fvar(xNA, f, wNA, na.rm = FALSE), wBY(xNA, f, wvar, wNA))
-  expect_equal(na20(fvar(xNA, f, wNA)), na20(wBY(xNA, f, wvar, wNA, na.rm = TRUE)))
-  expect_equal(na20(fvar(m, g, wdatNA)), na20(wBY(m, gf, wvar, wdatNA, na.rm = TRUE)))
-  expect_equal(fvar(m, g, wdatNA, na.rm = FALSE), wBY(m, gf, wvar, wdatNA))
-  expect_equal(fvar(mNA, g, wdatNA, na.rm = FALSE),  wBY(mNA, gf, wvar, wdatNA))
-  expect_equal(na20(fvar(mNA, g, wdatNA)), na20(wBY(mNA, gf, wvar, wdatNA, na.rm = TRUE)))
-  expect_equal(na20(fvar(mtcars, g, wdatNA)), na20(wBY(mtcars, gf, wvar, wdatNA, na.rm = TRUE)))
-  expect_equal(fvar(mtcars, g, wdatNA, na.rm = FALSE), wBY(mtcars, gf, wvar, wdatNA))
-  expect_equal(fvar(mtcNA, g, wdatNA, na.rm = FALSE), wBY(mtcNA, gf, wvar, wdatNA))
-  expect_equal(na20(fvar(mtcNA, g, wdatNA)), na20(wBY(mtcNA, gf, wvar, wdatNA, na.rm = TRUE)))
+  expect_equal(na20(fvar(x, f, wNA)), na20(BY(x, f, wvar, wNA, na.rm = TRUE)))
+  expect_equal(fvar(x, f, wNA, na.rm = FALSE), BY(x, f, wvar, wNA))
+  expect_equal(fvar(xNA, f, wNA, na.rm = FALSE), BY(xNA, f, wvar, wNA))
+  expect_equal(na20(fvar(xNA, f, wNA)), na20(BY(xNA, f, wvar, wNA, na.rm = TRUE)))
+  expect_equal(na20(fvar(m, g, wdatNA)), na20(BY(m, gf, wvar, wdatNA, na.rm = TRUE)))
+  expect_equal(fvar(m, g, wdatNA, na.rm = FALSE), BY(m, gf, wvar, wdatNA))
+  expect_equal(fvar(mNA, g, wdatNA, na.rm = FALSE),  BY(mNA, gf, wvar, wdatNA))
+  expect_equal(na20(fvar(mNA, g, wdatNA)), na20(BY(mNA, gf, wvar, wdatNA, na.rm = TRUE)))
+  expect_equal(na20(fvar(mtcars, g, wdatNA)), na20(BY(mtcars, gf, wvar, wdatNA, na.rm = TRUE)))
+  expect_equal(fvar(mtcars, g, wdatNA, na.rm = FALSE), BY(mtcars, gf, wvar, wdatNA))
+  expect_equal(fvar(mtcNA, g, wdatNA, na.rm = FALSE), BY(mtcNA, gf, wvar, wdatNA))
+  expect_equal(na20(fvar(mtcNA, g, wdatNA)), na20(BY(mtcNA, gf, wvar, wdatNA, na.rm = TRUE)))
 })
 
 test_that("fvar performs numerically stable", {
@@ -440,7 +434,7 @@ test_that("fvar with direct algorithm performs like base::var", {
   # expect_equal(fvar(mtcNA, g, stable.algo = FALSE), BY(mtcNA, g, bvar, na.rm = TRUE)) # failed on arch i386 CMD check and patched-solaris-x86
 })
 
-test_that("fvar with with direct algorithm and weights performs as intended (unbiased)", {
+test_that("fvar with direct algorithm and weights performs as intended (unbiased)", {
   expect_equal(fvar(c(2,2,4,5,5,5), stable.algo = FALSE), fvar(c(2,4,5), w = c(2,1,3), stable.algo = FALSE))
   expect_equal(fvar(c(2,2,4,5,5,5), na.rm = FALSE, stable.algo = FALSE), fvar(c(2,4,5), w = c(2,1,3), na.rm = FALSE), stable.algo = FALSE)
   expect_equal(fvar(c(2.456,2.456,4.123,5.009,5.009,5.009), stable.algo = FALSE), fvar(c(2.456,4.123,5.009), w = c(2,1,3), stable.algo = FALSE))
@@ -532,18 +526,18 @@ test_that("fvar with weights performs like wvar (defined above)", {
   expect_equal(fvar(mtcars, w = wdat, na.rm = FALSE, stable.algo = FALSE), dapply(mtcars, wvar, wdat))
   expect_equal(fvar(mtcNA, w = wdat, na.rm = FALSE, stable.algo = FALSE), dapply(mtcNA, wvar, wdat))
   expect_equal(fvar(mtcNA, w = wdat, stable.algo = FALSE), dapply(mtcNA, wvar, wdat, na.rm = TRUE))
-  expect_equal(fvar(x, f, w, stable.algo = FALSE), wBY(x, f, wvar, w))
-  expect_equal(fvar(x, f, w, na.rm = FALSE, stable.algo = FALSE), wBY(x, f, wvar, w))
-  expect_equal(fvar(xNA, f, w, na.rm = FALSE, stable.algo = FALSE), wBY(xNA, f, wvar, w))
-  expect_equal(na20(fvar(xNA, f, w, stable.algo = FALSE)), na20(wBY(xNA, f, wvar, w, na.rm = TRUE)))
-  expect_equal(fvar(m, g, wdat, stable.algo = FALSE), wBY(m, gf, wvar, wdat))
-  expect_equal(fvar(m, g, wdat, na.rm = FALSE, stable.algo = FALSE), wBY(m, gf, wvar, wdat))
-  expect_equal(fvar(mNA, g, wdat, na.rm = FALSE),  wBY(mNA, gf, wvar, wdat))
-  expect_equal(na20(fvar(mNA, g, wdat, stable.algo = FALSE)), na20(wBY(mNA, gf, wvar, wdat, na.rm = TRUE)))
-  expect_equal(fvar(mtcars, g, wdat, stable.algo = FALSE), wBY(mtcars, gf, wvar, wdat))
-  expect_equal(fvar(mtcars, g, wdat, na.rm = FALSE, stable.algo = FALSE), wBY(mtcars, gf, wvar, wdat))
-  expect_equal(fvar(mtcNA, g, wdat, na.rm = FALSE, stable.algo = FALSE), wBY(mtcNA, gf, wvar, wdat))
-  expect_equal(na20(fvar(mtcNA, g, wdat, stable.algo = FALSE)), na20(wBY(mtcNA, gf, wvar, wdat, na.rm = TRUE)))
+  expect_equal(fvar(x, f, w, stable.algo = FALSE), BY(x, f, wvar, w))
+  expect_equal(fvar(x, f, w, na.rm = FALSE, stable.algo = FALSE), BY(x, f, wvar, w))
+  expect_equal(fvar(xNA, f, w, na.rm = FALSE, stable.algo = FALSE), BY(xNA, f, wvar, w))
+  expect_equal(na20(fvar(xNA, f, w, stable.algo = FALSE)), na20(BY(xNA, f, wvar, w, na.rm = TRUE)))
+  expect_equal(fvar(m, g, wdat, stable.algo = FALSE), BY(m, gf, wvar, wdat))
+  expect_equal(fvar(m, g, wdat, na.rm = FALSE, stable.algo = FALSE), BY(m, gf, wvar, wdat))
+  expect_equal(fvar(mNA, g, wdat, na.rm = FALSE),  BY(mNA, gf, wvar, wdat))
+  expect_equal(na20(fvar(mNA, g, wdat, stable.algo = FALSE)), na20(BY(mNA, gf, wvar, wdat, na.rm = TRUE)))
+  expect_equal(fvar(mtcars, g, wdat, stable.algo = FALSE), BY(mtcars, gf, wvar, wdat))
+  expect_equal(fvar(mtcars, g, wdat, na.rm = FALSE, stable.algo = FALSE), BY(mtcars, gf, wvar, wdat))
+  expect_equal(fvar(mtcNA, g, wdat, na.rm = FALSE, stable.algo = FALSE), BY(mtcNA, gf, wvar, wdat))
+  expect_equal(na20(fvar(mtcNA, g, wdat, stable.algo = FALSE)), na20(BY(mtcNA, gf, wvar, wdat, na.rm = TRUE)))
   # missing weights
   expect_equal(fvar(NA, w = NA, stable.algo = FALSE), wvar(NA, NA))
   expect_equal(fvar(NA, w = NA, na.rm = FALSE, stable.algo = FALSE), wvar(NA, NA))
@@ -566,18 +560,18 @@ test_that("fvar with weights performs like wvar (defined above)", {
   expect_equal(fvar(mtcars, w = wdatNA, na.rm = FALSE, stable.algo = FALSE), dapply(mtcars, wvar, wdatNA))
   expect_equal(fvar(mtcNA, w = wdatNA, na.rm = FALSE, stable.algo = FALSE), dapply(mtcNA, wvar, wdatNA))
   expect_equal(fvar(mtcNA, w = wdatNA, stable.algo = FALSE), dapply(mtcNA, wvar, wdatNA, na.rm = TRUE))
-  expect_equal(na20(fvar(x, f, wNA, stable.algo = FALSE)), na20(wBY(x, f, wvar, wNA, na.rm = TRUE)))
-  expect_equal(fvar(x, f, wNA, na.rm = FALSE, stable.algo = FALSE), wBY(x, f, wvar, wNA))
-  expect_equal(fvar(xNA, f, wNA, na.rm = FALSE, stable.algo = FALSE), wBY(xNA, f, wvar, wNA))
-  expect_equal(na20(fvar(xNA, f, wNA, stable.algo = FALSE)), na20(wBY(xNA, f, wvar, wNA, na.rm = TRUE)))
-  expect_equal(na20(fvar(m, g, wdatNA, stable.algo = FALSE)), na20(wBY(m, gf, wvar, wdatNA, na.rm = TRUE)))
-  expect_equal(fvar(m, g, wdatNA, na.rm = FALSE, stable.algo = FALSE), wBY(m, gf, wvar, wdatNA))
-  expect_equal(fvar(mNA, g, wdatNA, na.rm = FALSE, stable.algo = FALSE),  wBY(mNA, gf, wvar, wdatNA))
-  expect_equal(na20(fvar(mNA, g, wdatNA, stable.algo = FALSE)), na20(wBY(mNA, gf, wvar, wdatNA, na.rm = TRUE)))
-  expect_equal(na20(fvar(mtcars, g, wdatNA, stable.algo = FALSE)), na20(wBY(mtcars, gf, wvar, wdatNA, na.rm = TRUE)))
-  expect_equal(fvar(mtcars, g, wdatNA, na.rm = FALSE, stable.algo = FALSE), wBY(mtcars, gf, wvar, wdatNA))
-  expect_equal(fvar(mtcNA, g, wdatNA, na.rm = FALSE, stable.algo = FALSE), wBY(mtcNA, gf, wvar, wdatNA))
-  expect_equal(na20(fvar(mtcNA, g, wdatNA, stable.algo = FALSE)), na20(wBY(mtcNA, gf, wvar, wdatNA, na.rm = TRUE)))
+  expect_equal(na20(fvar(x, f, wNA, stable.algo = FALSE)), na20(BY(x, f, wvar, wNA, na.rm = TRUE)))
+  expect_equal(fvar(x, f, wNA, na.rm = FALSE, stable.algo = FALSE), BY(x, f, wvar, wNA))
+  expect_equal(fvar(xNA, f, wNA, na.rm = FALSE, stable.algo = FALSE), BY(xNA, f, wvar, wNA))
+  expect_equal(na20(fvar(xNA, f, wNA, stable.algo = FALSE)), na20(BY(xNA, f, wvar, wNA, na.rm = TRUE)))
+  expect_equal(na20(fvar(m, g, wdatNA, stable.algo = FALSE)), na20(BY(m, gf, wvar, wdatNA, na.rm = TRUE)))
+  expect_equal(fvar(m, g, wdatNA, na.rm = FALSE, stable.algo = FALSE), BY(m, gf, wvar, wdatNA))
+  expect_equal(fvar(mNA, g, wdatNA, na.rm = FALSE, stable.algo = FALSE),  BY(mNA, gf, wvar, wdatNA))
+  expect_equal(na20(fvar(mNA, g, wdatNA, stable.algo = FALSE)), na20(BY(mNA, gf, wvar, wdatNA, na.rm = TRUE)))
+  expect_equal(na20(fvar(mtcars, g, wdatNA, stable.algo = FALSE)), na20(BY(mtcars, gf, wvar, wdatNA, na.rm = TRUE)))
+  expect_equal(fvar(mtcars, g, wdatNA, na.rm = FALSE, stable.algo = FALSE), BY(mtcars, gf, wvar, wdatNA))
+  expect_equal(fvar(mtcNA, g, wdatNA, na.rm = FALSE, stable.algo = FALSE), BY(mtcNA, gf, wvar, wdatNA))
+  expect_equal(na20(fvar(mtcNA, g, wdatNA, stable.algo = FALSE)), na20(BY(mtcNA, gf, wvar, wdatNA, na.rm = TRUE)))
 })
 
 test_that("fvar with direct algorithm performs numerically stable", {
@@ -610,7 +604,7 @@ test_that("fvar with direct algorithm performs numerically stable", {
   expect_true(all_obj_equal(replicate(50, fvar(mtcNA, g, stable.algo = FALSE), simplify = FALSE)))
 })
 
-test_that("fvar with with direct algorithm and complete weights performs numerically stable", {
+test_that("fvar with direct algorithm and complete weights performs numerically stable", {
   expect_true(all_obj_equal(replicate(50, fvar(1, w = 1, stable.algo = FALSE), simplify = FALSE)))
   expect_true(all_obj_equal(replicate(50, fvar(NA, w = 1, stable.algo = FALSE), simplify = FALSE)))
   expect_true(all_obj_equal(replicate(50, fvar(NA, w = 1, na.rm = FALSE, stable.algo = FALSE), simplify = FALSE)))
@@ -640,7 +634,7 @@ test_that("fvar with with direct algorithm and complete weights performs numeric
   expect_true(all_obj_equal(replicate(50, fvar(mtcNA, g, wdat, stable.algo = FALSE), simplify = FALSE)))
 })
 
-test_that("fvar with with direct algorithm and missing weights performs numerically stable", {
+test_that("fvar with direct algorithm and missing weights performs numerically stable", {
   expect_true(all_obj_equal(replicate(50, fvar(1, w = NA, stable.algo = FALSE), simplify = FALSE)))
   expect_true(all_obj_equal(replicate(50, fvar(NA, w = NA, stable.algo = FALSE), simplify = FALSE)))
   expect_true(all_obj_equal(replicate(50, fvar(NA, w = NA, na.rm = FALSE, stable.algo = FALSE), simplify = FALSE)))
@@ -695,7 +689,7 @@ test_that("fvar with direct algorithm handles special values in the right way",
   expect_equal(fvar(c(FALSE,FALSE), na.rm = FALSE, stable.algo = FALSE), 0)
 })
 
-test_that("fvar with with direct algorithm and weights handles special values in the right way", {
+test_that("fvar with direct algorithm and weights handles special values in the right way", {
   expect_equal(fvar(NA, w = 1, stable.algo = FALSE), NA_real_)
   expect_equal(fvar(NaN, w = 1, stable.algo = FALSE), NA_real_)
   expect_equal(fvar(Inf, w = 1, stable.algo = FALSE), NA_real_)
